////////////////
////////////////
///PXD - PXBE///
////////////////
////////////////

///PXBE shader release v1.1 (v21.02.09)///
///Do not use any of the code without permission///
///Even if you got my permission do not remove any thing that starts with /// 
///everything marked with /// is explaining what the code does or is some random text - PXD///

#include "Global_functions/tMap.hlsl"
#include "ShaderConstants.fxh"
#include "util.fxh"

struct PS_Input
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif
float4 fog = FOG_COLOR;
#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

   float2 FogControl = FOG_CONTROL;

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif
///PXDShaders - All code below this mark and above the other is IMPORTANT///

///water color///
#ifdef BLEND
if(PSInput.color.b>PSInput.color.r){
diffuse.rgb += float3(0.0,0.5,20.0);
diffuse.a *= float3(0.5,0.5,0.5);}
#endif
///

///adjust the color of the world lighting here///
float WLRED = 1.6;
float WLGRN = 1.6;
float WLBLU = 1.6;
float3 PXDWorld_color_lighting = float3(WLRED,WLGRN,WLBLU);

///adjust the torch lighting color here///
float TLred   = 1.0;
float TLgreen = 0.2;
float TLblue  = -0.5;
float3 PXDtlCLR = float3(TLred,TLgreen,TLblue);

///Shadow///

///shadow colors///
float3 sh   = float3(0.74,0.74,0.80);
float3 sh2  = float3(0.82,0.82,0.88);
float3 shL  = float3(0.25,0.15,0.0);
///

///shadow positions///
float sPOS1 = 0.88; float sPOS2 = 0.879;
float sPOS3 = 0.61; float sPOS4 = 0.60;
///

///Under///
if(PSInput.uv1.y<sPOS1)
{diffuse.rgb *= sh+PSInput.uv1.x*shL;}
if(PSInput.uv1.y<sPOS2)
{diffuse.rgb *= sh2+PSInput.uv1.x*shL;}
///
float3 WorldLGHTPXD = PXDWorld_color_lighting;
///Side///
if(PSInput.color.a!=0.0){ 
if(PSInput.color.g<sPOS3)
{diffuse.rgb *=sh+PSInput.uv1.x*shL;}
if(PSInput.color.g<sPOS4)
{diffuse.rgb *=sh2+PSInput.uv1.x*shL;}
}
///
float3 WorldRGB   = WorldLGHTPXD;
float3 WorldLight = WorldRGB;
float3 PXD_TL_RGB = PXDtlCLR;

///World Lighting///
diffuse.rgb *= WorldLight;

///Tmap///
diffuse.rgb = TMap(diffuse.rgb);

///torch lighting///
float3 TorchRGB = PXD_TL_RGB;
diffuse.rgb+=diffuse.rgb*(TorchRGB*2.0)*pow(PSInput.uv1.x*1.1,4.6);

///ALL code above this and blow the other mark is IMPORTANT///
#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}